'use strict';

const http = require('http');

module.exports = {
  port: 8088,
  hostname: '192.168.76.2'
};

module.exports.readFile = function(path, cb) {
  try {
    var req = http.request({
      hostname: module.exports.hostname,
      port: module.exports.port,
      path: '/',
      method: 'POST',
      headers: {
        'Content-Type': 'plain/text',
        'Content-Length': path.length,
        'Tmpdisk-Request-Type': 'read'
      }
    }, function(res) {
      var data = '';

      res.on('data', function(chunk) {
        data += String(chunk);
      });

      res.on('end', function() {
        if (res.statusCode === 500 || res.statusCode === 403 || res.statusCode === 400) {
          return cb(new Error(data), null);
        }
        cb(null, data);
      });
    });
    req.on('error', function(e) {
      cb(new Error('Error reading file.'), null);
    });
    req.write(path);
    req.end();
  } catch(e) {
    cb(new Error('Unknown error.'), null);
  }
}

module.exports.writeFile = function(path, data, encoding, cb) {
  if (typeof encoding === 'function') {
    cb = encoding;
    encoding = null;
  }
  encoding = encoding || 'utf8';
  try {
    var req = http.request({
      hostname: module.exports.hostname,
      port: module.exports.port,
      path: '/',
      method: 'POST',
      headers: {
        'Content-Type': 'plain/text',
        'Content-Length': data.length,
        'Tmpdisk-Request-File-Path': path,
        'Tmpdisk-Request-File-Encoding': encoding,
        'Tmpdisk-Request-Type': 'write'
      }
    }, function(res) {
      var data = '';

      res.on('data', function(chunk) {
        data += String(chunk);
      });

      res.on('end', function() {
        if (res.statusCode === 500 || res.statusCode === 403 || res.statusCode === 400) {
          return cb(new Error(data));
        }
        cb(null);
      });
    });
    req.on('error', function(e) {
      cb(new Error('Error writing file.'));
    });
    req.write(data);
    req.end();
  } catch(e) {
    cb(new Error('Unknown error.'));
  }
}

module.exports.readdir = function(path, cb) {
  try {
    var req = http.request({
      hostname: module.exports.hostname,
      port: module.exports.port,
      path: '/',
      method: 'POST',
      headers: {
        'Content-Type': 'plain/text',
        'Content-Length': path.length,
        'Tmpdisk-Request-Type': 'read'
      }
    }, function(res) {
      var data = '';

      res.on('data', function(chunk) {
        data += String(chunk);
      });

      res.on('end', function() {
        if (res.statusCode === 500 || res.statusCode === 403 || res.statusCode === 400) {
          return cb(new Error(data), null);
        }
        try {
          cb(null, JSON.parse(data));
        } catch(e) {
          cb(new Error('Error parsing data'), null);
        }
      });
    });
    req.on('error', function(e) {
      cb(new Error('Error reading directory.'), null);
    });
    req.write(path);
    req.end();
  } catch(e) {
    cb(new Error('Unknown error.'), null);
  }
}

module.exports.unlink = function(path, cb) {
  try {
    var req = http.request({
      hostname: module.exports.hostname,
      port: module.exports.port,
      path: '/',
      method: 'POST',
      headers: {
        'Content-Type': 'plain/text',
        'Content-Length': path.length,
        'Tmpdisk-Request-Type': 'delete'
      }
    }, function(res) {
      var data = '';

      res.on('data', function(chunk) {
        data += String(chunk);
      });

      res.on('end', function() {
        if (res.statusCode === 500 || res.statusCode === 403 || res.statusCode === 400) {
          return cb(new Error(data), null);
        }
        cb(null, data);
      });
    });
    req.on('error', function(e) {
      cb(new Error('Error removing file/directory.'), null);
    });
    req.write(path);
    req.end();
  } catch(e) {
    cb(new Error('Unknown error.'), null);
  }
}

module.exports.rmdir = module.exports.unlink;

module.exports.appendFile = function(path, data, encoding, cb) {
  if (typeof encoding === 'function') {
    cb = encoding;
    encoding = null;
  }
  encoding = encoding || 'utf8';
  try {
    var req = http.request({
      hostname: module.exports.hostname,
      port: module.exports.port,
      path: '/',
      method: 'POST',
      headers: {
        'Content-Type': 'plain/text',
        'Content-Length': data.length,
        'Tmpdisk-Request-File-Path': path,
        'Tmpdisk-Request-File-Encoding': encoding,
        'Tmpdisk-Request-Type': 'append'
      }
    }, function(res) {
      var data = '';

      res.on('data', function(chunk) {
        data += String(chunk);
      });

      res.on('end', function() {
        if (res.statusCode === 500 || res.statusCode === 403 || res.statusCode === 400) {
          return cb(new Error(data));
        }
        cb(null);
      });
    });
    req.on('error', function(e) {
      cb(new Error('Error writing file.'));
    });
    req.write(data);
    req.end();
  } catch(e) {
    cb(new Error('Unknown error.'));
  }
}

global.tmpdiskLoaded = true;
